@@ -19,20 +19,14 @@
 
 #include "kio_desktop.h"
 
-#include <KApplication>
-#include <KCmdLineArgs>
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KDirNotify>
-#include <KGlobalSettings>
-#include <KStandardDirs>
-#include <KGlobal>
-#include <KUrl>
-#include <kdeversion.h>
 
 #include <kio/udsentry.h>
 #include <kio_version.h>
 
+#include <QCoreApplication>
 #include <QFile>
 #include <QDBusInterface>
 #include <QDesktopServices>
@@ -45,8 +39,7 @@ extern "C"
     {
         // necessary to use other kio slaves
         QCoreApplication app(argc, argv);
-        KComponentData("kio_desktop", "kdelibs4");
-        KLocale::global();
+        app.setApplicationName("kio_desktop");
 
         // start the slave
         DesktopProtocol slave(argv[1], argv[2], argv[3]);
@@ -79,24 +72,6 @@ void DesktopProtocol::checkLocalInstall()
 
     const QDir desktopDir(desktopPath);
     bool desktopIsEmpty;
-    bool newRelease;
-
-    // Check if we have a new KDE release
-    KConfig config(QStringLiteral("kio_desktoprc"));
-    KConfigGroup cg(&config, "General");
-    QString version = cg.readEntry("Version", "0.0.0");
-    int major = version.section('.', 0, 0).toInt();
-    int minor = version.section('.', 1, 1).toInt();
-    int release = version.section('.', 2, 2).toInt();
-
-    if (KDE_MAKE_VERSION(major, minor, release) < KDE::version()) {
-        const QString version = QString::number(KDE::versionMajor()) + '.' +
-                                QString::number(KDE::versionMinor()) + '.' +
-                                QString::number(KDE::versionRelease());
-        cg.writeEntry("Version", version);
-        newRelease = true;
-    } else 
-        newRelease = false;
 
     // Create the desktop folder if it doesn't exist
     if (!desktopDir.exists()) {
@@ -115,40 +90,20 @@ void DesktopProtocol::checkLocalInstall()
                     desktopPath + "/trash.desktop");
  
         // Copy the desktop links
-        const QStringList links = KGlobal::dirs()->findAllResources("data", QStringLiteral("kio_desktop/DesktopLinks/*"),
-                                                                    KStandardDirs::NoDuplicates);
-        foreach (const QString &link, links) {
-            KDesktopFile file(link);
-            if (!file.desktopGroup().readEntry("Hidden", false))
-                QFile::copy(link, desktopPath + link.mid(link.lastIndexOf('/')));
-        }
-    } else if (newRelease) {
-        // Update the icon name in the .directory file to the FDO naming spec
-        const QString directoryFile = desktopPath + "/.directory";
-        if (QFile::exists(directoryFile)) {
-             KDesktopFile file(directoryFile);
-             if (file.readIcon() == QLatin1String("desktop"))
-                 file.desktopGroup().writeEntry("Icon", "user-desktop");
-        } else
-             QFile::copy(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kio_desktop/directory.desktop")), directoryFile);
-  
-        // Update the home icon to the FDO naming spec
-        const QString homeLink = desktopPath + "/Home.desktop";
-        if (QFile::exists(homeLink)) {
-            KDesktopFile home(homeLink);
-            const QString icon = home.readIcon();
-            if (icon == QLatin1String("kfm_home") || icon == QLatin1String("folder_home"))
-                home.desktopGroup().writeEntry("Icon", "user-home");
+        QSet<QString> links;
+        const auto dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kio_desktop/DesktopLinks"), QStandardPaths::LocateDirectory);
+        for (const auto &dir : dirs) {
+            const auto fileNames = QDir(dir).entryList({QStringLiteral("*.desktop")});
+            for (const auto &file : fileNames) {
+                links += file;
+            }
         }
 
-        // Update the trash icon to the FDO naming spec  
-        const QString trashLink = desktopPath + "/trash.desktop";
-        if (QFile::exists(trashLink)) {
-            KDesktopFile trash(trashLink);
-            if (trash.readIcon() == QLatin1String("trashcan_full"))
-                trash.desktopGroup().writeEntry("Icon", "user-trash-full");
-            if (trash.desktopGroup().readEntry("EmptyIcon") == QLatin1String("trashcan_empty"))
-                trash.desktopGroup().writeEntry("EmptyIcon", "user-trash");
+        foreach (const QString &link, links) {
+            const auto fullPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kio_desktop/DesktopLinks/%1").arg(link));
+            KDesktopFile file(fullPath);
+            if (!file.desktopGroup().readEntry("Hidden", false))
+                QFile::copy(fullPath, QStringLiteral("%1/%2").arg(desktopPath, link));
         }
     }
 #endif
@@ -165,7 +120,7 @@ void DesktopProtocol::listDir(const QUrl &url)
 {
     KIO::ForwardingSlaveBase::listDir(url);
 
-    KUrl actual;
+    QUrl actual;
     rewriteUrl(url, actual);
 
     QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/desktopnotifier"), QStringLiteral("org.kde.DesktopNotifier"));
@@ -178,12 +133,12 @@ QString DesktopProtocol::desktopFile(KIO::UDSEntry &entry) const
     if (name == QLatin1String(".") || name == QLatin1String(".."))
         return QString();
 
-    KUrl url = processedUrl();
-    url.addPath(name);
+    QUrl url = processedUrl();
+    url.setPath(QStringLiteral("%1/%2").arg(url.path(), name));
 
     if (entry.isDir()) {
-        url.addPath(QStringLiteral(".directory"));
-        if (!KStandardDirs::exists(url.path()))
+        url.setPath(QStringLiteral("%1/.directory").arg(url.path()));
+        if (!QFileInfo::exists(url.path()))
             return QString();
 
         return url.path();
