@@ -73,9 +73,8 @@ extern "C" int Q_DECL_EXPORT kdemain(int argc, char *argv[])
 
   KAboutData::setApplicationData(aboutData);
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
   QGuiApplication::setFallbackSessionManagementEnabled(false);
-#endif
+
   auto disableSessionManagement = [](QSessionManager &sm) {
       sm.setRestartHint(QSessionManager::RestartNever);
   };
