@@ -23,8 +23,8 @@
 #include <Plasma/Theme>
 #include <QTimer>
 
-#include "plasmaquick/containmentview.h"
-#include "plasmaquick/configview.h"
+#include <PlasmaQuick/ContainmentView>
+#include <PlasmaQuick/ConfigView>
 
 class ShellCorona;
 
@@ -77,6 +77,13 @@ class PanelView : public PlasmaQuick::ContainmentView
      */
     Q_PROPERTY(int distance READ distance WRITE setDistance NOTIFY distanceChanged)
 
+    /**
+     * support NoBackground in order to disable blur/contrast effects and remove
+     * the panel shadows
+     * @since 5.9
+     */    
+    Q_PROPERTY(Plasma::Types::BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints NOTIFY backgroundHintsChanged)
+
     /**
      * The borders that should have a shadow
      * @since 5.7
@@ -101,7 +108,7 @@ public:
         LetWindowsCover, /** always visible, windows will go over the panel, no area reserved */
         WindowsGoBelow /** always visible, windows will go under the panel, no area reserved */
     };
-    Q_ENUMS(VisibilityMode)
+    Q_ENUM(VisibilityMode)
 
     explicit PanelView(ShellCorona *corona, QScreen *targetScreen = 0, QWindow *parent = 0);
     ~PanelView() override;
@@ -131,6 +138,9 @@ public:
     int distance() const;
     void setDistance(int dist);
 
+    Plasma::Types::BackgroundHints backgroundHints() const;
+    void setBackgroundHints(Plasma::Types::BackgroundHints hint);
+
     Plasma::FrameSvg::EnabledBorders enabledBorders() const;
 
     VisibilityMode visibilityMode() const;
@@ -168,6 +178,7 @@ Q_SIGNALS:
     void maximumLengthChanged();
     void minimumLengthChanged();
     void distanceChanged();
+    void backgroundHintsChanged();
     void enabledBordersChanged();
 
     //QWindow does not have a property for screen. Adding this property requires re-implementing the signal
@@ -198,6 +209,7 @@ private:
     bool containmentContainsPosition(const QPointF &point) const;
     QPointF positionAdjustedForContainment(const QPointF &point) const;
     void setupWaylandIntegration();
+    void visibilityModeToWayland();
     bool edgeActivated() const;
     void updateEnabledBorders();
     bool canSetStrut() const;
@@ -208,6 +220,7 @@ private:
     int m_contentLength;
     int m_distance;
     int m_thickness;
+    bool m_initCompleted;
     Qt::Alignment m_alignment;
     QPointer<PlasmaQuick::ConfigView> m_panelConfigView;
     ShellCorona *m_corona;
@@ -218,6 +231,7 @@ private:
     QTimer m_unhideTimer;
     //only for the mask, not to actually paint
     Plasma::FrameSvg *m_background;
+    Plasma::Types::BackgroundHints m_backgroundHints;
     Plasma::FrameSvg::EnabledBorders m_enabledBorders = Plasma::FrameSvg::AllBorders;
     KWayland::Client::PlasmaShellSurface *m_shellSurface;
     QPointer<QScreen> m_lastScreen;
